using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
// Import SqlClient and System.IO namespaces
using System.Data.SqlClient;
using System.IO;

public partial class _SelectMssql : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        // This code will only be executed if the page load not is a post back.
        if (!Page.IsPostBack)
        {
            // Get values from session variables and store these values in hidden fields.
            HiddenCompanyID.Value = Convert.ToString(Session["CompanyID"]);
            HiddenYearID.Value = Convert.ToString(Session["YearID"]);
            HiddenCompanyName.Value = Convert.ToString(Session["CompanyName"]);
        }
    }

    protected void BtnCheckUserName_Click(object sender, EventArgs e)
    {
        // This code is to select just one value "UserName" to "lookupUserName" from a table by 
        // using "ExecuteScalar()" as the command

        // Declare a string variable and a assign a empty value to it.
        string lookupUserName = string.Empty;

        // Declare variables for a connection string and the SELECT statement.
        string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string SelectUser = "SELECT UserName FROM Users WHERE UserName = @UserName";

        // Create a SqlConnection. The using block is used to call dispose (close) automatically even 
        // if there are an exception.
        using (SqlConnection cn = new SqlConnection(ConnString))
        {
            // Create a SqlCommand.
            SqlCommand cmd = new SqlCommand(SelectUser, cn);

            // Add parameters for the WHERE clause, parameters makes it harder for hackers to modify
            // sql statements.
            cmd.Parameters.AddWithValue("@UserName", txtUserName.Text);

            // The Try/Catch/Finally block is used to handle exceptions in the application.
            try
            {
                // Open the SqlConnection, will be closed when the using block has reached its end.
                cn.Open();

                // Execute the SELECT command and fetch UserName field into the lookupUserName string.
                lookupUserName = Convert.ToString(cmd.ExecuteScalar());

            }
            catch (Exception ex)
            {
                // Catch a general exception.
                Response.Write(ex.Message);
            }
            finally
            {
                // Dispose the SqlCommand.
                cmd.Dispose();
            }

            // Compare the supplied username to the selected username from the Users table and print to a label.
            if (lookupUserName == txtUserName.Text)
            {
                UserNameLabel.Text = "Username already exists in the database";
            }
            else
            {
                UserNameLabel.Text = "Username does not exist in the database";
            }
        }
    }

    protected void btnGetData_Click(object sender, EventArgs e)
    {
        // This code is to select rows from the Assets table with a SqlDataReader where the companyid matches

        string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "SELECT Place, AssetOwnID, AssetName, VerNumber, Supplier FROM Assets WHERE CompanyID "
        + "= @CompanyID AND SoldScraped = '0' ORDER BY Place";

        // Create a SqlConnection. The using block is used to call dispose (close) automatically even if 
        // there are an exception.
        using (SqlConnection cn = new SqlConnection(ConnString))
        {
            // Create a SqlCommand.
            SqlCommand cmd = new SqlCommand(sql, cn);
            
            // Create a SqlDataReader.
            SqlDataReader reader = null;

            // Add parameters for the WHERE clause.
            cmd.Parameters.AddWithValue("@CompanyID", HiddenCompanyID.Value);

            // The Try/Catch/Finally block is used to handle exceptions in the application.
            try
            {
                // Open the SqlConnection.
                cn.Open();

                // Execute the SELECT statement and assign the result to the reader.
                reader = cmd.ExecuteReader();

                // Set the datasource for the repeater and bind data to it.
                SelectToRepeaterDataReader.DataSource = reader;
                SelectToRepeaterDataReader.DataBind();
            }
            catch (Exception ex)
            {
                // Catch a general exception.
                Response.Write(ex.Message);
            }
            finally
            {
                // Dispose the SqlCommand.
                cmd.Dispose();

                // Close the reader to avoid memory leakage.
                if (reader != null)
                    reader.Close();
            }
        }
    }

    protected void btnGetUserData_Click(object sender, EventArgs e)
    {
        // This code shows how to select one row from the Users table where the username is equal to the entered username by
        // using a SqlDataReader.

        // Declare variables for a connection string and a SELECT statement.
        string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "SELECT * FROM Users WHERE Username = @Username";

        // Create a SqlConnection. The using block is used to call dispose (close) automatically 
        // even if there are an exception. 
        using (SqlConnection cn = new SqlConnection(ConnString))
        {
            // Create a SqlCommand.
            SqlCommand cmd = new SqlCommand(sql, cn);

            // Create a SqlDataReader.
            SqlDataReader reader = null;

            // Add parameters for the WHERE clause.
            cmd.Parameters.AddWithValue("@Username", txtUserName2.Text);

            // The Try/Catch/Finally block is used to handle exceptions in the application.
            try
            {
                // Open the SqlConnection.
                cn.Open();

                // We use SqlDataReader and just want to select one single row, the Using block
                // is used to call dispose (close) automatically even if there are an exception.
                reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                // A do while loop for the reader.
                while (reader.Read())
                {
                    txtUserID.Text = reader["UserID"].ToString();
                    txtUserName.Text = reader["Username"].ToString();
                    txtCompanyName.Text = reader["Company"].ToString();
                    txtOrganisationNumber.Text = reader["OrgNumber"].ToString();
                    txtContact.Text = reader["Contact"].ToString();
                    txtAttention.Text = reader["Attention"].ToString();
                    txtAdress.Text = reader["Adress"].ToString();
                    txtPostalCode.Text = reader["PostalCode"].ToString();
                    txtCity.Text = reader["City"].ToString();
                    txtCountry.Text = reader["Country"].ToString();
                }
            }
            catch (Exception ex)
            {
                // Catch a general exception.
                Response.Write(ex.Message);
            }
            finally
            {
                // Dispose the SqlCommand.
                cmd.Dispose();

                // Close the reader to avoid memory leakage.
                if (reader != null)
                    reader.Close();
            }
        }
    }

    protected void BtnSelectAssets_Click(object sender, EventArgs e)
    {
        // This is code to select data to a dataset with two tables so that we kan make groups and calculate values
        // Asset accounts in parentrepeater and assets inside childrepeater.

        // Declare variables for a connection string and two SELECT statements.
        string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "SELECT AssetTypeID, AssetAccount, TypeName FROM AssetType WHERE CompanyID = @CompanyID "
        + "GROUP BY AssetTypeID, AssetAccount, TypeName ORDER BY AssetAccount ASC";
        string sql2 = "SELECT A.AssetOwnID, A.AssetType, A.AssetName, A.PurchaseDate, A.PurchasePrice, "
        + "SUM(COALESCE(SumIB, 0)) As YearIB, SUM(COALESCE(SumYearDep,0)) As YearDep, SUM(COALESCE(SumIB, 0) "
        + "+ COALESCE(SumYearDep,0)) As SumAckDeprication, SUM(A.PurchasePrice - (COALESCE(SumIB, 0) + "
        + "COALESCE(SumYearDep,0))) As AccountedValue FROM Assets AS A LEFT OUTER JOIN (SELECT AssetID, "
        + "SUM(IBValue) As SumIB FROM IBDeprication WHERE YearID = @YearID GROUP BY AssetID) As IB ON A.AssetID "
        + "= IB.AssetID LEFT OUTER JOIN (SELECT AssetID, SUM(DepricationValue) As SumYearDep FROM Deprication "
        + "WHERE YearID = @YearID GROUP BY AssetID) AS D ON A.AssetID = D.AssetID WHERE A.CompanyID = @CompanyID "
        + "AND A.SoldScraped = '0' OR A.DateSold > @Slutdatum GROUP BY A.AssetOwnID, A.AssetType, A.AssetName, "
        + "A.PurchaseDate, A.PurchasePrice ORDER BY A.PurchaseDate ASC";

        // Create a SqlConnection. The using block is used to call dispose (close) automatically even if there 
        // are an exception.
        using (SqlConnection cn = new SqlConnection(ConnString))
        {
            // Create the first SqlDataAdapter.
            SqlDataAdapter ad = new SqlDataAdapter();

            // Create the first SqlCommand.
            SqlCommand cmd = new SqlCommand(sql, cn);

            // Create the second SqlDataAdapter.
            SqlDataAdapter ad2 = new SqlDataAdapter();

            // Create the second SqlCommand.
            SqlCommand cmd2 = new SqlCommand(sql2, cn);

            // Create a new dataset, a dataset can contain several datatables that are related to each other.
            DataSet ds = new DataSet();

            // Add parameters.
            cmd.Parameters.AddWithValue("@CompanyID", HiddenCompanyID.Value);
            cmd2.Parameters.AddWithValue("@YearID", HiddenYearID.Value);
            cmd2.Parameters.AddWithValue("@CompanyID", HiddenCompanyID.Value);
            cmd2.Parameters.AddWithValue("@Slutdatum", txtEndDate.Text);

            // The Try/Catch/Finally block is used to handle exceptions in the application.
            try
            {
                // Open the SqlConnection.
                cn.Open();

                // Fill dataset with assettypes as the first table.
                ad.SelectCommand = cmd;
                ad.Fill(ds, "AssetTypes");

                // Fill dataset with assets as the second table.
                ad2.SelectCommand = cmd2;
                ad2.Fill(ds, "Assets");

                // Create the relation bewtween the AssetTypes and Assets tables.
                ds.Relations.Add("assetsrelation", ds.Tables["AssetTypes"].Columns["AssetTypeID"], ds.Tables["Assets"].Columns["AssetType"]);

                // Calculate subtotals
                DataColumn PPTotal = new DataColumn("PPTotalt", System.Type.GetType("System.Decimal"), "Sum(Child(assetsrelation).PurchasePrice)");
                PPTotal.AutoIncrement = false;
                PPTotal.ReadOnly = true;
                DataColumn IBTotal = new DataColumn("IBtotalt", System.Type.GetType("System.Decimal"), "Sum(Child(assetsrelation).YearIB)");
                IBTotal.AutoIncrement = false;
                IBTotal.ReadOnly = true;
                DataColumn YearTotal = new DataColumn("YearDepTotalt", System.Type.GetType("System.Decimal"), "Sum(Child(assetsrelation).YearDep)");
                YearTotal.AutoIncrement = false;
                YearTotal.ReadOnly = true;
                DataColumn AckDepTotal = new DataColumn("AckDepTotalt", System.Type.GetType("System.Decimal"), "Sum(Child(assetsrelation).SumAckDeprication)");
                AckDepTotal.AutoIncrement = false;
                AckDepTotal.ReadOnly = true;
                DataColumn AccValueTotal = new DataColumn("AccValueTotalt", System.Type.GetType("System.Decimal"), "Sum(Child(assetsrelation).AccountedValue)");
                AccValueTotal.AutoIncrement = false;
                AccValueTotal.ReadOnly = true;

                // Add columns to the AssetTypes table.
                ds.Tables["AssetTypes"].Columns.Add(PPTotal);
                ds.Tables["AssetTypes"].Columns.Add(IBTotal);
                ds.Tables["AssetTypes"].Columns.Add(YearTotal);
                ds.Tables["AssetTypes"].Columns.Add(AckDepTotal);
                ds.Tables["AssetTypes"].Columns.Add(AccValueTotal);

                // Bind the AssetTypes table to the parentrepeater control, and call DataBind. Data for the 
                // childrepeater will be retrieved in Select-mssql.aspx from the dataset.
                parentRepeater.DataSource = ds.Tables["AssetTypes"];
                Page.DataBind();
            }
            catch (Exception ex)
            {
                // Catch a general exception.
                Response.Write(ex.Message);
            }
            finally
            {
                // Dispose of SqlDataAdapters, SqlCommands and the DataSet.
                ad.Dispose();
                ad2.Dispose();
                cmd.Dispose();
                cmd2.Dispose();
                ds.Dispose();
            }
        }
    }
}
